/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.mif.konfigdaten;

import java.util.ArrayList;
import java.util.List;

import org.geotools.feature.FeatureType;

import com.vividsolutions.jts.geom.Geometry;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.mif.mif.MifObjekt;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;
import de.bsvrz.sys.funclib.kbgen.ProxyDataDescription;

/**
 * Klasse zur Repr&auml;sentation eines Konfigurationsobjektes vom Typ Linie.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class MifGeoLinie extends AbstraktKonfigurationsObjekt {
	/**
	 * Konstruktor mit PID und Name.
	 *
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 * @param davobjekttyp
	 *            Zeiltyp im DAV
	 * @param geometrie
	 *            Geometrie
	 * @param featuretype
	 *            FeatureTyp
	 */
	public MifGeoLinie(final String pid, final String name, final String davobjekttyp, final Geometry geometrie,
			final FeatureType featuretype) {
		super(pid, name, davobjekttyp, geometrie, featuretype);
		setKoordinaten(MifObjekt.getFeatureKoordinatenWGS84(featuretype, geometrie));
	}

	/**
	 * Versorgt alle Attribute und erzeugt ein {@link ProxyConfigurationObject}.
	 *
	 * @param dav
	 *            Datenverteiler-Verbindung
	 *
	 * @return {@link ProxyConfigurationObject}
	 * @throws ImportException
	 *             bei Ausnahmen
	 */
	@Override
	@SuppressWarnings("nls")
	public List<ProxyConfigurationObject> getProxyObjekt(final ClientDavConnection dav) throws ImportException {
		final List<ProxyConfigurationObject> ret = new ArrayList<ProxyConfigurationObject>();

		final ProxyConfigurationObject pco = new ProxyConfigurationObject(pid, name,
				dav.getDataModel().getType(davobjektyp));

		Data data = createData(dav, "atg.linienKoordinaten", "asp.eigenschaften");

		data.setToDefault();
		final Data.NumberArray x = data.getScaledArray("x");
		final Data.NumberArray y = data.getScaledArray("y");

		x.setLength(wgs84.size());
		y.setLength(wgs84.size());

		for (int i = 0; i < wgs84.size(); i++) {
			x.getValue(i).set(wgs84.get(i).getLaenge());
			y.getValue(i).set(wgs84.get(i).getBreite());
		}
		pco.addConfigurationData(new ProxyDataDescription(data, null));

		data = createData(dav, "atg.linie", "asp.eigenschaften");
		data.getScaledValue("Länge").set(laenge);
		pco.addConfigurationData(new ProxyDataDescription(data, null));

		ret.add(pco);
		return ret;
	}
}
