/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.mif.mif;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import org.geotools.data.DataStore;
import org.geotools.data.DefaultQuery;
import org.geotools.data.FeatureReader;
import org.geotools.data.Query;
import org.geotools.data.Transaction;
import org.geotools.data.mif.BC_MIFDataStore;
import org.geotools.data.mif.BC_MIFDataStoreFactory;
import org.geotools.feature.AttributeType;
import org.geotools.feature.Feature;
import org.geotools.feature.GeometryAttributeType;
import org.geotools.feature.IllegalAttributeException;

import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.PrecisionModel;

import de.bsvrz.kex.mif.konfigdaten.ImportException;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Realisiert das Modul MifDaten der SWE.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class MifDaten implements MifDatenInterface, MifDatenInfo {
	/**
	 * DebugLogger für Debug-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	private HashMap<String, Object> params;

	private final String uri = null;
	/**
	 * Default SRID.
	 */
	public static final int SRID = 26591;
	/**
	 * GeometryFactory.
	 */
	public static final GeometryFactory GEOMFACTORY = new GeometryFactory(
			new PrecisionModel(PrecisionModel.FLOATING_SINGLE), MifDaten.SRID);

	/**
	 * .
	 */
	public static final String COORDSYSCLAUSE = "Earth Projection 8, 87, \"m\", 9, 0, 0.9996, 1500000, 0 Bounds (-6746230.6469, -9998287.38389) (9746230.6469, 9998287.38389)"; //$NON-NLS-1$

	private final List<Feature> featureliste = new ArrayList<Feature>();

	/*
	 * (Kein Javadoc)
	 * 
	 * @see de.bsvrz.kex.mif.mif.MifDatenInterface#getinfo()
	 */
	@Override
	public MifDatenInfo getinfo() {
		return this;
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see
	 * de.bsvrz.kex.mif.mif.MifDatenInterface#getobjekte(de.bsvrz.kex.mif.mif.
	 * MifTyp)
	 */
	@Override
	public MifObjekt[] getobjekte(final MifTyp filter) {
		final MifObjekt[] array = new MifObjekt[featureliste.size()];

		int index = 0;

		for (final Iterator<Feature> fi = featureliste.iterator(); fi.hasNext(); index++) {
			array[index] = new MifObjekt(fi.next());
		}

		return array;
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see de.bsvrz.kex.mif.mif.MifDatenInterface#importiere(java.lang.String)
	 */
	@Override
	@SuppressWarnings("nls")
	public void importiere(final String pfadname) throws ImportException {

		// teste file
		// File mfile = new File(pfadname);

		MifDaten.DEBUG.info("Die MIF-Datenbasis '" + pfadname + "' wird importiert");

		params = new HashMap<String, Object>();

		params.put("dbtype", "mif");
		params.put("path", pfadname);

		if (uri != null) {
			params.put("namespace", uri);
		}

		final String geomType = "untyped";

		params.put(BC_MIFDataStore.PARAM_FIELDCASE, "upper");
		params.put(BC_MIFDataStore.PARAM_GEOMNAME, "the_geom");
		params.put(BC_MIFDataStore.PARAM_GEOMTYPE, geomType);

		// params.put(MIFDataStore.PARAM_GEOMFACTORY, MIFTestUtils.geomFactory);
		params.put(BC_MIFDataStore.PARAM_SRID, new Integer(MifDaten.SRID));

		// params.put(MIFDataStore.HCLAUSE_COORDSYS,
		// MIFTestUtils.coordsysClause);

		final BC_MIFDataStoreFactory mdf = new BC_MIFDataStoreFactory();

		try {
			final DataStore ds = mdf.createDataStore(params);

			final String[] tn = ds.getTypeNames();

			for (final String element : tn) {
				final Query query = new DefaultQuery(element);
				final FeatureReader reader = ds.getFeatureReader(query, Transaction.AUTO_COMMIT);
				try {
					while (reader.hasNext()) {
						featureliste.add(reader.next());
					}
				} catch (final NoSuchElementException e) {
					throw new ImportException(e.getMessage());
				} catch (final IllegalAttributeException e) {
					throw new ImportException(e.getMessage());
				} finally {
					reader.close();
				}
			}
		} catch (final IOException e) {
			if (e instanceof FileNotFoundException) {
				throw new ImportException("Die MIF-Datenbasis '" + pfadname + "' konnte nicht gefunden werden!");
			}

			e.printStackTrace();
			throw new ImportException(e.getMessage());
		} 

		MifDaten.DEBUG.info("Die MIF-Datenbasis '" + pfadname + "' wurde importiert");
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see de.bsvrz.kex.mif.mif.MifDatenInfo#getAnzahlObjekte()
	 */
	@Override
	public int getAnzahlObjekte() {
		return featureliste.size();
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see de.bsvrz.kex.mif.mif.MifDatenInfo#getMifTypen()
	 */
	@Override
	public MifTyp[] getMifTypen() {
		final HashSet<MifTyp> typen = new HashSet<MifTyp>();

		for (final Feature f : featureliste) {
			try {
				typen.add(MifTyp.bestimmeTyp(f));
			} catch (final ImportException e) {
				MifDaten.DEBUG.error(e.getMessage());
			}
		}

		final MifTyp[] typarray = new MifTyp[typen.size()];
		int index = 0;

		final Iterator<MifTyp> it = typen.iterator();

		while (it.hasNext()) {
			typarray[index++] = it.next();
		}

		return typarray;
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see de.bsvrz.kex.mif.mif.MifDatenInfo#getSpaltenNamen()
	 */
	@Override
	public String[] getSpaltenNamen() {
		final Feature f = featureliste.get(0);
		final String[] spalten = new String[f.getFeatureType().getAttributeCount()];

		int index = 0;
		for (final AttributeType a : f.getFeatureType().getAttributeTypes()) {
			// skip "the_geom"
			if (a instanceof GeometryAttributeType) {
				continue;
			}
			spalten[index++] = a.getName();
		}

		return spalten;
	}

}
